package com.cn.esermis.dpld.business;

import com.alibaba.fastjson2.JSON;
import com.alibaba.fastjson2.JSONObject;
import com.cn.esermis.coding.PollutionGas;
import com.cn.esermis.constant.Constant;
import com.cn.esermis.datawarn.common.ExceptionDataDispose;
import com.cn.esermis.dpld.entity.*;
import com.cn.esermis.dpld.model.OutletAndFactorVo;
import com.cn.esermis.dpld.service.*;
import com.cn.esermis.model.CP;
import com.cn.esermis.model.FactorModel;
import lombok.extern.log4j.Log4j2;
import org.apache.logging.log4j.core.util.UuidUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Component;

import javax.annotation.PostConstruct;
import javax.annotation.Resource;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Optional;

@Component
@Log4j2
public class MonitorBusiness {

    @Resource
    private IWaterMonitorMinuteRecordService waterMonitorMinuteRecordService;
    @Resource
    private IWaterMonitorHourRecordService waterMonitorHourRecordService;
    @Resource
    private IAirMonitorMinuteRecordService airMonitorMinuteRecordService;
    @Autowired(required = true)
    private ExceptionDataDispose exceptionDataDispose;
    @Resource
    private IDataAcquisitionInstrumentService dataAcquisitionInstrumentService;
    @Resource
    private IExcepLogService excepLogService;
    @Resource
    private IAirMonitorHourRecordService airMonitorHourRecordService;
    @Resource
    private IPostMarkingService postMarkingService;
    @Resource
    private RedisTemplate<String, Object> redisTemplate;



    @PostConstruct
    public void loadData() {

        List<OutletAndFactorVo> selectlist = dataAcquisitionInstrumentService.selectlist();
        redisTemplate.opsForValue().set("dataAcquisitionInstruments", JSON.toJSONString(selectlist));
        // 处理读取到的数据
    }



    /**
     * 处理消息入库
     *
     * @param msg
     */
    public void inboundOperations(JSONObject msg) {
        // 消息入库
        if (msg == null) {
            return;
        }
        FactorModel factorModel = new FactorModel().factorModel(msg);
        List<CP> cp = factorModel.getCP();
        log.info("入库数据：{}", msg);
        try {
            if (factorModel.getST().equals(Constant.ST_AIR)) {
                // 气体检测
                if (factorModel.getCN().equals(Constant.MINUTE)) {
                    // 分钟数据入库
                    List<AirMonitorMinuteRecord> listData = new ArrayList<>();
                    cp.forEach(x -> {
                        AirMonitorMinuteRecord airMonitorMinuteRecord = new AirMonitorMinuteRecord();
                        airMonitorMinuteRecord.setQn(factorModel.getQN());
                        airMonitorMinuteRecord.setSt(factorModel.getST());
                        airMonitorMinuteRecord.setCn(factorModel.getCN());
                        airMonitorMinuteRecord.setPw(factorModel.getPW());
                        airMonitorMinuteRecord.setMn(factorModel.getMN());
                        airMonitorMinuteRecord.setFlag(factorModel.getFlag());
                        airMonitorMinuteRecord.setDataTime(factorModel.getDataTime());
                        airMonitorMinuteRecord.setFactor(x.getFactor());
                        airMonitorMinuteRecord.setMin(x.getMin());
                        airMonitorMinuteRecord.setMax(x.getMax());
                        airMonitorMinuteRecord.setAvg(x.getAvg());
                        airMonitorMinuteRecord.setCou(x.getCou());
                        airMonitorMinuteRecord.setFactorFlag(x.getFlag());
                        airMonitorMinuteRecord.setZsAvg(x.getZsAvg());
                        airMonitorMinuteRecord.setZsMax(x.getZsMax());
                        airMonitorMinuteRecord.setZsMin(x.getZsMin());
                        airMonitorMinuteRecord.setRtd(x.getRtd());
                        try {
                            airMonitorMinuteRecord.setWorkCondition(sendMessage(factorModel.getDataTime(),factorModel.getMN()));
                        } catch (ParseException e) {
                            throw new RuntimeException(e);
                        }
                        try {
                            airMonitorMinuteRecord.setIsExceed(exceptionDataDispose.ExceedData(factorModel.getMN(), x.getFactor(), x.getAvg()!=null?x.getAvg().doubleValue():0));
                            airMonitorMinuteRecord.setIsLose(exceptionDataDispose.LoseData(x.getMax()!=null?x.getMax().doubleValue():0,
                                    x.getMin()!=null?x.getMin().doubleValue():0,
                                    x.getAvg()!=null?x.getAvg().doubleValue():0));
                            if ((airMonitorMinuteRecord.getIsExceed()!=null && !airMonitorMinuteRecord.getIsExceed().equals("1") )||
                                    (airMonitorMinuteRecord.getIsLose()!=null && !airMonitorMinuteRecord.getIsLose().equals("1") )||
                                    (airMonitorMinuteRecord.getIsSix()!=null && !airMonitorMinuteRecord.getIsSix().equals("1"))){
                                saveExcepLog(x, airMonitorMinuteRecord);
                            }
                        } catch (Exception e) {
//                            throw  new RuntimeException(e);
                            log.error("调用 ExceptionDataDispose1 方法异常{} 当前参数为:{}",e,airMonitorMinuteRecord);
                        }
                        listData.add(airMonitorMinuteRecord);
                    });
                    airMonitorMinuteRecordService.saveBatch(listData);
                }
//                else if (factorModel.getCN().equals(Constant.REAL)) {
//                    // 分钟数据入库
//                    List<AirMonitorMinuteRecord> listData = new ArrayList<>();
//                    cp.forEach(x -> {
//                        AirMonitorMinuteRecord airMonitorMinuteRecord = new AirMonitorMinuteRecord();
//                        airMonitorMinuteRecord.setQn(factorModel.getQN());
//                        airMonitorMinuteRecord.setSt(factorModel.getST());
//                        airMonitorMinuteRecord.setCn(factorModel.getCN());
//                        airMonitorMinuteRecord.setPw(factorModel.getPW());
//                        airMonitorMinuteRecord.setMn(factorModel.getMN());
//                        airMonitorMinuteRecord.setFlag(factorModel.getFlag());
//                        airMonitorMinuteRecord.setDataTime(factorModel.getDataTime());
//                        airMonitorMinuteRecord.setFactor(x.getFactor());
//                        airMonitorMinuteRecord.setMin(x.getMin());
//                        airMonitorMinuteRecord.setMax(x.getMax());
//                        airMonitorMinuteRecord.setAvg(x.getAvg());
//                        airMonitorMinuteRecord.setCou(x.getCou());
//                        airMonitorMinuteRecord.setFactorFlag(x.getFlag());
//                        airMonitorMinuteRecord.setZsAvg(x.getZsAvg());
//                        airMonitorMinuteRecord.setZsMax(x.getZsMax());
//                        airMonitorMinuteRecord.setZsMin(x.getZsMin());
//                        airMonitorMinuteRecord.setRtd(x.getRtd());
//                        try {
//                            airMonitorMinuteRecord.setWorkCondition(sendMessage(factorModel.getDataTime(),factorModel.getMN()));
//                        } catch (ParseException e) {
//                            throw new RuntimeException(e);
//                        }
//                        try {
//                            airMonitorMinuteRecord.setIsExceed(exceptionDataDispose.ExceedData(factorModel.getMN(), x.getFactor(), x.getAvg()!=null?x.getAvg().doubleValue():0));
//                            airMonitorMinuteRecord.setIsLose(exceptionDataDispose.LoseData(x.getMax()!=null?x.getMax().doubleValue():0,
//                                    x.getMin()!=null?x.getMin().doubleValue():0,
//                                    x.getAvg()!=null?x.getAvg().doubleValue():0));
//                            if ((airMonitorMinuteRecord.getIsExceed()!=null && !airMonitorMinuteRecord.getIsExceed().equals("Y") )||
//                                    (airMonitorMinuteRecord.getIsLose()!=null && !airMonitorMinuteRecord.getIsLose().equals("Y") )||
//                                    (airMonitorMinuteRecord.getIsSix()!=null && !airMonitorMinuteRecord.getIsSix().equals("Y"))){
//                                saveExcepLog(x, airMonitorMinuteRecord);
//                            }
//                        } catch (Exception e) {
//                            log.error("调用 ExceptionDataDispose2 方法异常{} 当前参数为:{}",e,airMonitorMinuteRecord);
//                        }
//                        listData.add(airMonitorMinuteRecord);
//                    });
//                    if (!listData.isEmpty()) {
//                        airMonitorMinuteRecordService.saveBatch(listData);
//                    }
//                }
                else if (factorModel.getCN().equals(Constant.HOUR)) {
                    // 小时数据入库
                    List<AirMonitorHourRecord> listData = new ArrayList<>();
                    cp.forEach(x -> {
                        AirMonitorHourRecord airMonitorHourRecord = new AirMonitorHourRecord();
                        airMonitorHourRecord.setQn(factorModel.getQN());
                        airMonitorHourRecord.setSt(factorModel.getST());
                        airMonitorHourRecord.setCn(factorModel.getCN());
                        airMonitorHourRecord.setPw(factorModel.getPW());
                        airMonitorHourRecord.setMn(factorModel.getMN());
                        airMonitorHourRecord.setFlag(factorModel.getFlag());
                        airMonitorHourRecord.setDataTime(factorModel.getDataTime());
                        airMonitorHourRecord.setFactor(x.getFactor());
                        airMonitorHourRecord.setMin(x.getMin());
                        airMonitorHourRecord.setMax(x.getMax());
                        airMonitorHourRecord.setAvg(x.getAvg());
                        airMonitorHourRecord.setCou(x.getCou());
                        airMonitorHourRecord.setFactorFlag(x.getFlag());
                        airMonitorHourRecord.setZsAvg(x.getZsAvg());
                        airMonitorHourRecord.setZsMax(x.getZsMax());
                        airMonitorHourRecord.setZsMin(x.getZsMin());
                        airMonitorHourRecord.setRtd(x.getRtd());
                        try {
                            airMonitorHourRecord.setWorkCondition(sendHoursMessage(factorModel.getDataTime(),factorModel.getMN()));
                        } catch (Exception e) {
                            throw new RuntimeException(e);
                        }
                        try {
                            airMonitorHourRecord.setIsExceed(exceptionDataDispose.ExceedData(factorModel.getMN(), x.getFactor(), x.getAvg()!=null?x.getAvg().doubleValue():0));
                            airMonitorHourRecord.setIsLose(exceptionDataDispose.LoseData(x.getMax()!=null?x.getMax().doubleValue():0,
                                    x.getMin()!=null?x.getMin().doubleValue():0,
                                    x.getAvg()!=null?x.getAvg().doubleValue():0));
                            airMonitorHourRecord.setIsSix(exceptionDataDispose.SixData(factorModel.getMN(), x.getFactor(),x.getAvg()!=null?x.getAvg().doubleValue():0, factorModel.getDataTime()));
//                            if (!airMonitorHourRecord.getIsExceed().equals("Y") || !airMonitorHourRecord.getIsLose().equals("Y") || !airMonitorHourRecord.getIsSix().equals("Y")){
                            if ((airMonitorHourRecord.getIsExceed()!=null && !airMonitorHourRecord.getIsExceed().equals("1") )||
                                    (airMonitorHourRecord.getIsLose()!=null && !airMonitorHourRecord.getIsLose().equals("1") )||
                                    (airMonitorHourRecord.getIsSix()!=null && !airMonitorHourRecord.getIsSix().equals("1"))){
                                saveExcepLog(x, airMonitorHourRecord);
                            }
                        } catch (Exception e) {
                            log.error("调用 ExceptionDataDispose3 方法异常{} 当前参数为:{}",e,airMonitorHourRecord);
                        }
                        listData.add(airMonitorHourRecord);
                    });
                    airMonitorHourRecordService.saveBatch(listData);
                }
            } else if (factorModel.getST().equals(Constant.ST_WATER)) {
                // 污水检测
                if (factorModel.getCN().equals(Constant.MINUTE)) {
                    // 分钟数据入库
                    List<WaterMonitorMinuteRecord> listData = new ArrayList<>();
                    cp.forEach(x -> {
                        WaterMonitorMinuteRecord waterMonitorMinuteRecord = new WaterMonitorMinuteRecord();
                        waterMonitorMinuteRecord.setQn(factorModel.getQN());
                        waterMonitorMinuteRecord.setSt(factorModel.getST());
                        waterMonitorMinuteRecord.setCn(factorModel.getCN());
                        waterMonitorMinuteRecord.setPw(factorModel.getPW());
                        waterMonitorMinuteRecord.setMn(factorModel.getMN());
                        waterMonitorMinuteRecord.setFlag(factorModel.getFlag());
                        waterMonitorMinuteRecord.setDataTime(factorModel.getDataTime());
                        waterMonitorMinuteRecord.setFactor(x.getFactor());
                        waterMonitorMinuteRecord.setMin(x.getMin());
                        waterMonitorMinuteRecord.setMax(x.getMax());
                        waterMonitorMinuteRecord.setAvg(x.getAvg());
                        waterMonitorMinuteRecord.setCou(x.getCou());
                        waterMonitorMinuteRecord.setFactorFlag(x.getFlag());
                        waterMonitorMinuteRecord.setRtd(x.getRtd());
                        try {
                            waterMonitorMinuteRecord.setWorkCondition(sendMessage(factorModel.getDataTime(),factorModel.getMN()));
                        } catch (ParseException e) {
                            throw new RuntimeException(e);
                        }
                        try {
                            waterMonitorMinuteRecord.setIsExceed(exceptionDataDispose.ExceedData(factorModel.getMN(), x.getFactor(), x.getAvg()!=null?x.getAvg().doubleValue():0));
                            waterMonitorMinuteRecord.setIsLose(exceptionDataDispose.LoseData(x.getMax()!=null?x.getMax().doubleValue():0,
                                    x.getMin()!=null?x.getMin().doubleValue():0,
                                    x.getAvg()!=null?x.getAvg().doubleValue():0));
                            if ((waterMonitorMinuteRecord.getIsExceed()!=null && !waterMonitorMinuteRecord.getIsExceed().equals("1") )||
                                    (waterMonitorMinuteRecord.getIsLose()!=null && !waterMonitorMinuteRecord.getIsLose().equals("1") )||
                                    (waterMonitorMinuteRecord.getIsSix()!=null && !waterMonitorMinuteRecord.getIsSix().equals("1"))){
                                saveExcepLog(x, waterMonitorMinuteRecord);
                            }
                        } catch (Exception e) {
                            log.error("调用 ExceptionDataDispose4 方法异常{} 当前参数为:{}",e,waterMonitorMinuteRecord);
                        }
                        listData.add(waterMonitorMinuteRecord);
                    });
                    waterMonitorMinuteRecordService.saveBatch(listData);
                }
//                else if (factorModel.getCN().equals(Constant.REAL)) {
//                    // 分钟数据入库
//                    List<WaterMonitorMinuteRecord> listData = new ArrayList<>();
//                    cp.forEach(x -> {
//                        WaterMonitorMinuteRecord waterMonitorMinuteRecord = new WaterMonitorMinuteRecord();
//                        waterMonitorMinuteRecord.setQn(factorModel.getQN());
//                        waterMonitorMinuteRecord.setSt(factorModel.getST());
//                        waterMonitorMinuteRecord.setCn(factorModel.getCN());
//                        waterMonitorMinuteRecord.setPw(factorModel.getPW());
//                        waterMonitorMinuteRecord.setMn(factorModel.getMN());
//                        waterMonitorMinuteRecord.setFlag(factorModel.getFlag());
//                        waterMonitorMinuteRecord.setDataTime(factorModel.getDataTime());
//                        waterMonitorMinuteRecord.setFactor(x.getFactor());
//                        waterMonitorMinuteRecord.setMin(x.getMin());
//                        waterMonitorMinuteRecord.setMax(x.getMax());
//                        waterMonitorMinuteRecord.setAvg(x.getAvg());
//                        waterMonitorMinuteRecord.setCou(x.getCou());
//                        waterMonitorMinuteRecord.setFactorFlag(x.getFlag());
//                        waterMonitorMinuteRecord.setRtd(x.getRtd());
//                        try {
//                            waterMonitorMinuteRecord.setWorkCondition(sendMessage(factorModel.getDataTime(),factorModel.getMN()));
//                        } catch (ParseException e) {
//                            throw new RuntimeException(e);
//                        }
//                        try {
//                            waterMonitorMinuteRecord.setIsExceed(exceptionDataDispose.ExceedData(factorModel.getMN(), x.getFactor(), x.getAvg()!=null?x.getAvg().doubleValue():0));
//                            waterMonitorMinuteRecord.setIsLose(exceptionDataDispose.LoseData(x.getMax()!=null?x.getMax().doubleValue():0,
//                                    x.getMin()!=null?x.getMin().doubleValue():0,
//                                    x.getAvg()!=null?x.getAvg().doubleValue():0));
//                            if ((waterMonitorMinuteRecord.getIsExceed()!=null && !waterMonitorMinuteRecord.getIsExceed().equals("Y") )||
//                                    (waterMonitorMinuteRecord.getIsLose()!=null && !waterMonitorMinuteRecord.getIsLose().equals("Y") )||
//                                    (waterMonitorMinuteRecord.getIsSix()!=null && !waterMonitorMinuteRecord.getIsSix().equals("Y"))){
//                                saveExcepLog(x, waterMonitorMinuteRecord);
//                            }
//                        } catch (Exception e) {
//                            log.error("调用 ExceptionDataDispose5 方法异常{} 当前参数为:{}",e,waterMonitorMinuteRecord);
//                        }
//
//                        listData.add(waterMonitorMinuteRecord);
//                    });
//                    if (!listData.isEmpty()) {
//                        waterMonitorMinuteRecordService.saveBatch(listData);
//                    }
//                }
                else if
                (factorModel.getCN().equals(Constant.HOUR)) {
                    // 小时数据入库
                    List<WaterMonitorHourRecord> listData = new ArrayList<>();
                    cp.forEach(x -> {
                        WaterMonitorHourRecord waterMonitorHourRecord = new WaterMonitorHourRecord();
                        waterMonitorHourRecord.setQn(factorModel.getQN());
                        waterMonitorHourRecord.setSt(factorModel.getST());
                        waterMonitorHourRecord.setCn(factorModel.getCN());
                        waterMonitorHourRecord.setPw(factorModel.getPW());
                        waterMonitorHourRecord.setMn(factorModel.getMN());
                        waterMonitorHourRecord.setFlag(factorModel.getFlag());
                        waterMonitorHourRecord.setDataTime(factorModel.getDataTime());
                        waterMonitorHourRecord.setFactor(x.getFactor());
                        waterMonitorHourRecord.setMin(x.getMin());
                        waterMonitorHourRecord.setMax(x.getMax());
                        waterMonitorHourRecord.setAvg(x.getAvg());
                        waterMonitorHourRecord.setCou(x.getCou());
                        waterMonitorHourRecord.setFactorFlag(x.getFlag());
                        waterMonitorHourRecord.setRtd(x.getRtd());
                        try {
                            waterMonitorHourRecord.setWorkCondition(sendHoursMessage(factorModel.getDataTime(),factorModel.getMN()));
                        } catch (Exception e) {
                            throw new RuntimeException(e);
                        }
                        try {
                            waterMonitorHourRecord.setIsExceed(exceptionDataDispose.ExceedData(factorModel.getMN(), x.getFactor(), x.getAvg()!=null?x.getAvg().doubleValue():0));
                            waterMonitorHourRecord.setIsLose(exceptionDataDispose.LoseData(x.getMax()!=null?x.getMax().doubleValue():0,
                                    x.getMin()!=null?x.getMin().doubleValue():0,
                                    x.getAvg()!=null?x.getAvg().doubleValue():0));
                            waterMonitorHourRecord.setIsSix(exceptionDataDispose.SixData(factorModel.getMN(), x.getFactor(),x.getAvg()!=null?x.getAvg().doubleValue():0, factorModel.getDataTime()));
                            if ((waterMonitorHourRecord.getIsExceed()!=null && !waterMonitorHourRecord.getIsExceed().equals("1") )||
                                    (waterMonitorHourRecord.getIsLose()!=null && !waterMonitorHourRecord.getIsLose().equals("1") )||
                                    (waterMonitorHourRecord.getIsSix()!=null && !waterMonitorHourRecord.getIsSix().equals("1"))){
                                saveExcepLog(x, waterMonitorHourRecord);
                            }
                        } catch (Exception e) {
                            log.error("调用 ExceptionDataDispose6 方法异常{} 当前参数为:{}",e,waterMonitorHourRecord);
                        }
                        listData.add(waterMonitorHourRecord);
                    });
                    waterMonitorHourRecordService.saveBatch(listData);
                }
            }
        } catch (Exception e) {
//            throw new RuntimeException(e);
            log.error("插入数据库异常："+e);
        }
    }

    private void saveExcepLog(CP x, AirMonitorMinuteRecord record) {
        ExcepLog excepLog = new ExcepLog();
        Optional<OutletAndFactorVo> first = getloadData().stream().filter(d ->d.getFactor()!=null && d.getFactor().equals(x.getFactor())).findFirst();
        if (first.isPresent()) {
            excepLog.setOutletName(first.get().getOutletName());
            excepLog.setOutletCode(first.get().getOutletCode());
            excepLog.setMn(record.getMn());
            excepLog.setFactor(record.getFactor());
            excepLog.setExcepTime(new Date());
            excepLog.setSixLast(record.getIsSix());
            excepLog.setLoseData(record.getIsLose());
            excepLogService.save(excepLog);
        }
        if (record.getIsLose()!=null && !record.getIsLose().equals("1")){
            addExcepLog(record.getFactor(), "1");
        }
        if (record.getIsSix()!=null && !record.getIsSix().equals("1")){
            addExcepLog(record.getFactor(), "2");
        }
        if (record.getIsExceed()!=null && !record.getIsExceed().equals("1")){
            addExcepLog(record.getFactor(), "3");
        }
    }

    private void saveExcepLog(CP x, AirMonitorHourRecord record) {
        ExcepLog excepLog = new ExcepLog();
        Optional<OutletAndFactorVo> first = getloadData().stream().filter(d -> d.getFactor()!=null && d.getFactor().equals(x.getFactor())).findFirst();
        if (first.isPresent()) {
            excepLog.setOutletName(first.get().getOutletName());
            excepLog.setOutletCode(first.get().getOutletCode());
            excepLog.setMn(record.getMn());
            excepLog.setFactor(record.getFactor());
            excepLog.setExcepTime(new Date());
            excepLog.setSixLast(record.getIsSix());
            excepLog.setLoseData(record.getIsLose());
            excepLogService.save(excepLog);
        }
        if (record.getIsLose()!=null && !record.getIsLose().equals("1")){
            addExcepLog(record.getFactor(), "1");
        }
        if (record.getIsSix()!=null && !record.getIsSix().equals("1")){
            addExcepLog(record.getFactor(), "2");
        }
        if (record.getIsExceed()!=null && !record.getIsExceed().equals("1")){
            addExcepLog(record.getFactor(), "3");
        }
    }

    private void saveExcepLog(CP x, WaterMonitorMinuteRecord record) {
        ExcepLog excepLog = new ExcepLog();
        Optional<OutletAndFactorVo> first = getloadData().stream().filter(d ->d.getFactor()!=null &&  d.getFactor().equals(x.getFactor())).findFirst();
        if (first.isPresent()) {
            excepLog.setOutletName(first.get().getOutletName());
            excepLog.setOutletCode(first.get().getOutletCode());
            excepLog.setMn(record.getMn());
            excepLog.setFactor(record.getFactor());
            excepLog.setExcepTime(new Date());
            excepLog.setSixLast(record.getIsSix());
            excepLog.setLoseData(record.getIsLose());
            excepLogService.save(excepLog);
        }
        if (record.getIsLose()!=null && !record.getIsLose().equals("1")){
            addExcepLog(record.getFactor(), "1");
        }
        if (record.getIsSix()!=null && !record.getIsSix().equals("1")){
            addExcepLog(record.getFactor(), "2");
        }
        if (record.getIsExceed()!=null && !record.getIsExceed().equals("1")){
            addExcepLog(record.getFactor(), "3");
        }
    }
    private void saveExcepLog(CP x, WaterMonitorHourRecord record) {
        ExcepLog excepLog = new ExcepLog();
        Optional<OutletAndFactorVo> first = getloadData().stream().filter(d -> d.getFactor()!=null && d.getFactor().equals(x.getFactor())).findFirst();
        if (first.isPresent()) {
            excepLog.setOutletName(first.get().getOutletName());
            excepLog.setOutletCode(first.get().getOutletCode());
            excepLog.setMn(record.getMn());
            excepLog.setFactor(record.getFactor());
            excepLog.setExcepTime(new Date());
            excepLog.setSixLast(record.getIsSix());
            excepLog.setLoseData(record.getIsLose());
            excepLogService.save(excepLog);
        }
        if (record.getIsLose()!=null && !record.getIsLose().equals("1")){
            addExcepLog(record.getFactor(), "1");
        }
        if (record.getIsSix()!=null && !record.getIsSix().equals("1")){
            addExcepLog(record.getFactor(), "2");
        }
        if (record.getIsExceed()!=null && !record.getIsExceed().equals("1")){
            addExcepLog(record.getFactor(), "3");
        }
    }

    public Integer sendMessage(String time,String mn) throws ParseException {
        SimpleDateFormat formatter = new SimpleDateFormat("yyyyMMddHHmmss");
        Object o = redisTemplate.opsForValue().get(mn);
        if (o != null){
            PreMarking preMarking = JSON.parseObject(o.toString(), PreMarking.class);
            Date date = formatter.parse(time);
            if (preMarking.getStartTime().getTime() <= date.getTime()){
                return preMarking.getWorkingCondition();
            }
        }
        return 1;
    }

    public Integer sendHoursMessage(String time,String mn){
        SimpleDateFormat formatter = new SimpleDateFormat("yyyyMMddHH");
        Object o = redisTemplate.opsForValue().get(mn);
        if (o != null){
            PreMarking preMarking = JSON.parseObject(o.toString(), PreMarking.class);
//            Date date = formatter.parse(time);
            String format = formatter.format(preMarking.getStartTime().getTime());
            if (time.contains(format)){
                return preMarking.getWorkingCondition();
            }
        }
        return 1;
    }
    /**
     * 获取工厂排口信息
     */
    private List<OutletAndFactorVo> getloadData() {
        List<OutletAndFactorVo>  dataAcquisitionInstruments = new ArrayList<>();
        Object o = redisTemplate.opsForValue().get("dataAcquisitionInstruments");
        if (o != null){
            dataAcquisitionInstruments = JSON.parseArray(o.toString(), OutletAndFactorVo.class);
        }else {
            dataAcquisitionInstruments = dataAcquisitionInstrumentService.selectlist();
            redisTemplate.opsForValue().set("dataAcquisitionInstruments", JSON.toJSONString(dataAcquisitionInstruments));
        }
        return dataAcquisitionInstruments;
    }
    /**
     * 添加事后处置单信息
     */
    public void addExcepLog(String factor,String dataType) {
        PostMarking postMarking = new PostMarking();
        Optional<OutletAndFactorVo> first = getloadData().stream().filter(d -> d.getFactor()!=null && d.getFactor().equals(factor)).findFirst();
        if (first.isPresent()) {
            postMarking.setOutletName(first.get().getOutletName());
            postMarking.setOutletNumber(first.get().getOutletCode());
            postMarking.setManagementNumber(String.valueOf(UuidUtil.getTimeBasedUuid()));
            postMarking.setMnCode(first.get().getMnCode());
            postMarking.setCompanyCode(first.get().getCompanyCode());
            postMarking.setDepartmentCode(first.get().getDepartmentCode());
            PollutionGas pollutionGas = PollutionGas.valueOf(factor);
            switch (dataType){
                case "1":
                    //判断是否为负值数据
                    postMarking.setManagementContent("数据异常:监测因子【"+pollutionGas.mean()+"("+pollutionGas.code()+")】采集数据为负数。");
                    break;
                case "2":
                    // //判断是否为六小时恒值数据
                    postMarking.setManagementContent("数据异常：监测因子【"+pollutionGas.mean()+"("+pollutionGas.code()+")】6小时恒值异常。");
                    break;
                case "3":
                    //瞬时值是否为超标数据
                    postMarking.setManagementContent("数据异常：监测因子【"+pollutionGas.mean()+"("+pollutionGas.code()+")】瞬时值超标。");
                    break;
            }
            postMarkingService.save(postMarking);
        }
    }

}
